from os.path import getmtime
from pathlib import Path

import structlog

log = structlog.get_logger()


def by_date_desc(paths: [Path]) -> list:
    return list(reversed(sorted(paths, key=getmtime)))


def keepn(directory: Path, n_keep: int, dryrun=False):
    paths = by_date_desc(directory.iterdir())
    keep, delete = paths[:n_keep], paths[n_keep:]
    log.info('keep', paths=[str(x) for x in sorted(keep, key=getmtime)])
    log.info('delete', paths=[str(x) for x in sorted(delete, key=getmtime)])
    if dryrun:
        log.info('DRYRUN. returning.')
        return
    else:
        for path in delete:
            path.unlink()



import json
from pathlib import Path


class JSONThing:
    def __init__(self, raw):
        self.raw = raw

    def __repr__(self):
        return str(self.raw)[:30]

    @property
    def id(self):
        return self.raw['id']

    @property
    def uid(self):
        return self.raw['uid']

    @property
    def title(self):
        return self.raw['title']


class Dashboard(JSONThing):
    def __init__(self, raw, base_url):
        super().__init__(raw)
        base = str(base_url).rstrip('/')
        self.url = f'{base}/d/{self.uid}'

    def export(self, path):
        path = Path(path)
        path.write_text(json.dumps(self.raw))



import gitlab
from gitlab.v4.objects import ProjectJob


class Gitlab:
    """
    Note the use of `iterator=True` to handle pagination
    https://python-gitlab.readthedocs.io/en/stable/api-usage.html#pagination.
    """

    PAGINATION_ARGS = {'iterator': True, 'per_page': 1000}

    def __init__(self, python_gitlab_instance):
        self.gl: gitlab.Gitlab = python_gitlab_instance

    @classmethod
    def from_ini(cls, name):
        return cls(gitlab.Gitlab.from_config(name))

    def jobs_of_project(self, project_id) -> [ProjectJob]:
        """
        Note that project_id can also be a string like "namespace/project". :)
        """
        p = self.gl.projects.get(project_id)
        return p.jobs.list(**self.PAGINATION_ARGS)
